diff -Naur ../../test10/lib/libdba/Makefile ./libdba/Makefile
--- ../../test10/lib/libdba/Makefile	Wed Dec 31 16:00:00 1969
+++ ./libdba/Makefile	Fri Nov 10 13:14:23 2000
@@ -0,0 +1,12 @@
+
+INCLUDE=	-I. -I../../linux/include
+
+DEBUG=-O3
+
+all: libdba.so
+
+libdba.so: aiolib.c
+	cc -shared -olibdba.so $(INCLUDE) aiolib.c
+
+install: libdba.so
+	install -m 0755 libdba.so $(INSTALLROOT)/lib
diff -Naur ../../test10/lib/libdba/README ./libdba/README
--- ../../test10/lib/libdba/README	Wed Dec 31 16:00:00 1969
+++ ./libdba/README	Fri Nov 10 13:13:57 2000
@@ -0,0 +1,134 @@
+/*
+ *
+ * libdba: Facilities which accelerate database performance.
+ * 	   Included are POSIX Asynchronous I/O,
+ *			Post-wait synchronization,
+ *			and a fast gettimeofday().
+ *
+ * Copyright 1999, Silicon Graphics, Inc.
+ *
+ * Written October 1999 by Rajagopal Ananthanarayanan (ananth)
+ *  			at Silicon Graphics, Inc.
+ */
+
+Asynchronous I/O Facility
+-------------------------
+
+Overview
+--------
+
+The asynchronous I/O (AIO) facility implements interfaces defined by the
+POSIX standard. The AIO facility implemented in the SGI Linux Environment
+1.1 differs from glibc implementation.  The  glibc version realizes
+asynchrony by employing slave threads to process the I/O requests. This
+implies that only as many I/O requests as the number of slaves are truly
+asynchronous at the device, since the slaves use the blocking system calls
+to service the I/O: when a slave thread is processing an I/O request,
+it is blocked in that I/O system call.
+
+The SGI AIO version is mostly implemented in the Linux kernel. To
+distinguish with the glibc version, we will use the term KAIO. In KAIO,
+when possible, I/O requests are implemented as split-phase I/O requests.
+With split-phase I/O, the initiating request (such as an aio_read) truly
+queues the I/O at the device as the first phase of the I/O request; a
+second phase of the I/O request, performed as part of the I/O completion,
+propagates results of the request.  The results may include the contents
+of the I/O buffer on a read, the number of bytes read or written, and
+any error status.  Thus, with split-phase I/O as much asynchrony as the
+device can support is actually achieved. However, not all file systems or
+devices readily support split-phase I/O. Currently, KAIO only supports
+split-phase I/O for file systems that employ generic_file_read() as
+their read routine (several file systems in current Linux kernel use
+generic_file_read, including popular ext2), and for all Character Disk
+Devices (Raw) which is also provided as part of SGI Linux Environment 1.1.
+For requests which are not split-phase, KAIO employs slave threads,
+similar to the glibc implementation.
+
+
+Using KAIO
+----------
+
+KAIO implements POSIX Asynchronous I/O interfaces, although formal
+compliance testing and branding have not been attempted yet.  All interfaces,
+such as aio_read(), aio_write(), aio_suspend(), aio_cancel(), aio_error(),
+aio_return(), and lio_listio() are supported. Further, all interfaces
+can use 64-bit offsets allowing greater than 2GB offsets, as discussed below.
+
+To use KAIO as opposed to glibc AIO, one has to include <linux/aio.h>,
+since glibc already defines a <aio.h>. Further, libdba.so has to be
+linked into the executable; the sources for libdba can be found in
+/usr/src/linux/lib/libdba. Finally, libdba.so needs to be installed
+under /lib on the machine where the executable is to run.  KAIO and
+glibc AIO cannot be intermixed; including <aio.h> and <linux/aio.h>
+would cause compilation errors, since both define, for instance, aiocb,
+the AIO control block.
+
+The kernel support for KAIO is enabled using the CONFIG_AIO option,
+as part of "General Setup". Further, CONFIG_AIO_MAX allows the maximum
+number of outstanding I/Os to be configurable, although the current
+default of 4096 should be sufficient for most environments.
+
+KAIO is designed to work with POSIX Threads (pthreads), or any programs
+that employ clone threads, as long as the clones are created with the
+CLONE_VM option.
+
+Finally, the number of slave threads can be changed using the environment
+variable AIO_NTHREADS. By default 4 threads (AIO_DEFAULT_THREADS in
+<linux/aio.h>) are created. All slaves have the prefix "kaiod-XXXXX" as
+their name in top(1) or pstree(1), where "XXXXX" is the pid of the
+parent which created the slaves; see BUGS about ps(1).
+
+
+Enabling 64-bit offsets
+-----------------------
+
+User code can turn on 64-bit offsets in aiocb by:
+
+      #define _FILE_OFFSET_BITS 64
+      #include <features.h>
+      #include <linux/aio.h>
+which will turn all aio_*() interfaces to operate with 64-bit offsets.
+More precisely, aio_offset will be a 64-bit quantitity (loff_t).
+
+Further, if _LARGEFILE64_SOURCE is defined in the above, it will enable
+LFS style explicit aio_*64() and lio_listio64() interfaces; these will
+accept aiocb64_t as their argument. If only _LARGEFILE64_SOURCE is
+defined, then aio_*() and lio_listio() will use 32-bit offsets.
+
+Note that if you include other system header files before including it
+as above, those headers may include <features.h>, which will turn the
+later inclusion of <features.h> into a null inclusion.  This will lead
+to the incorrect AIO interfaces.  To avoid this, one possibility is
+to define either or both of _FILE_OFFSET_BITS and _LARGEFILE64_SOURCE
+before including any system header file.
+
+As an alternate way of turning on 64-bit capability, AIO_FILE_OFFSET64
+enables 64-bit offsets, and AIO_LARGEFILE64 enables LFS interfaces.
+While this scheme does not involve <features.h> and its associated
+include-order problem, these definitions are understood only by
+KAIO code.
+
+The kernel always uses 64-bit offset values. If user code does not
+enable 64-bit offsets, the library (libdba.so) will suitably zero
+out the padding.
+
+Known Bugs
+----------
+
+ps(1) shows the slave threads with the name of the parent,
+although top(1) and pstree(1) show the name correctly.
+
+Resources
+---------
+
+(1) Where to find AIO man pages?
+
+	http://www.opengroup.org/public/pubs/online/7908799/xsh/aio.h.html
+
+Note: aio_fsync() is not required by POSIX, and currently
+not implemented in KAIO.
+	
+(2) Where to find LFS specifications?
+
+	http://ftp.sas.com/standards/large.file/
+
diff -Naur ../../test10/lib/libdba/aiolib.c ./libdba/aiolib.c
--- ../../test10/lib/libdba/aiolib.c	Wed Dec 31 16:00:00 1969
+++ ./libdba/aiolib.c	Fri Nov 10 13:13:57 2000
@@ -0,0 +1,242 @@
+/*
+ * Copyright 1999, Silicon Graphics, Inc.
+ *
+ * Written October 1999 by Rajagopal Ananthanarayanan (ananth)
+ *  			at Silicon Graphics, Inc.
+ */
+#define AIO_LARGEFILE64
+#include <linux/aio.h>
+#include <signal.h>
+#include <time.h>
+#include <sys/types.h>
+#include <bits/siginfo.h>
+#include <stdio.h>
+#include <unistd.h>
+#include <errno.h>
+#include <stdlib.h>
+#include <asm/unistd.h>
+#include <sched.h>
+
+
+_syscall5(int, aio,
+		int, cmd,
+		unsigned long, arg1,
+		unsigned long, arg2,
+		unsigned long, arg3,
+		unsigned long, arg4);
+
+static int aio_cpid[AIO_MAX_THREADS];
+
+
+int
+child(void *arg)
+{
+	/* child */
+	aio(AIOCMD_SLAVE, 0, 0, 0, 0);
+	exit(-1);
+}
+
+
+static void
+aio_init(int nslaves)
+{
+	int i;
+	char *cstack;
+
+	if (nslaves == 0) {
+		char *p;
+
+		/* get it from the environment */
+		if ((p = getenv(AIO_THREAD_ENVIRON)))
+			nslaves = strtol(p, 0, 0);
+		else
+			nslaves = AIO_DEFAULT_THREADS;
+	}
+
+	if (nslaves <= 0 || nslaves > AIO_MAX_THREADS) {
+		fprintf(stderr, "aio_init: Warning! nthreads %d not in "
+				"valid range 1 .. %d (see AIO_MAX_THREADS)\n",
+				nslaves, AIO_MAX_THREADS); 
+		fprintf(stderr, "aio_init: defaulting to %d threads "
+				"(see AIO_DEFAULT_THREADS)\n",
+				AIO_DEFAULT_THREADS);
+		nslaves = AIO_DEFAULT_THREADS;
+	}
+
+	for (i = 0; i < nslaves; i++) {
+		cstack = (char *) malloc(64 * 1024);
+		cstack += (32 * 1024);
+		cstack = (char *)((unsigned long)cstack & 0xfffff000);
+
+		/*
+		aio_cpid[i] =
+			aio(AIOCMD_SLAVE, 0, 0, 0, 0);
+		*/
+		aio_cpid[i] = clone(child, cstack,
+			SIGCHLD|CLONE_VM|CLONE_FS|CLONE_FILES, 0);
+		if (aio_cpid[i] < 0) {
+			perror("clone");
+			exit(-1);
+		}
+	}
+}
+
+int
+aio_read64(aiocb64_t *aiocb)
+{
+	int ret;
+
+	ret = aio(AIOCMD_READ, (unsigned long)aiocb, 0, 0, 0);
+	if (ret < 0 && errno == ENOTCONN) {
+		aio_init(0);
+		ret = aio(AIOCMD_READ, (unsigned long)aiocb, 0, 0, 0);
+	}
+	return ret;
+}
+
+int
+aio_write64(aiocb64_t *aiocb)
+{
+	int ret;
+
+	ret = aio(AIOCMD_WRITE, (unsigned long)aiocb, 0, 0, 0);
+	if (ret < 0 && errno == ENOTCONN) {
+		aio_init(0);
+		ret = aio(AIOCMD_WRITE, (unsigned long)aiocb, 0, 0, 0);
+	}
+	return ret;
+}
+
+
+int
+aio_cancel64(int fildes, aiocb64_t *aiocb)
+{
+	int ret;
+
+	ret = aio(AIOCMD_CANCEL, (unsigned long)fildes,
+			(unsigned long)aiocb, 0, 0);
+	if (ret < 0 && errno == ENOTCONN) {
+		aio_init(0);
+		ret = aio(AIOCMD_CANCEL, (unsigned long)fildes,
+				(unsigned long)aiocb, 0, 0);
+	}
+	return ret;
+}
+
+int
+aio_suspend64(const aiocb64_t *const list[], int nent, const struct timespec *timeout)
+{
+	int ret;
+
+	ret = aio(AIOCMD_SUSPEND, (unsigned long)list, (unsigned long)nent,
+			(unsigned long)timeout, 0);
+	if (ret < 0 && errno == ENOTCONN) {
+		aio_init(0);
+		ret = aio(AIOCMD_SUSPEND, (unsigned long)list,
+				(unsigned long)nent,
+				(unsigned long)timeout, 0);
+	}
+	return ret;
+}
+
+ssize_t
+aio_return64(aiocb64_t *aiocb)
+{
+	return(aiocb->aio_reserved[3]);
+}
+
+int
+lio_listio64(int mode, aiocb64_t *const list[], int nent, sigevent_t *sig)
+{
+	int ret;
+
+	ret = aio(AIOCMD_LIST_IO,
+			(unsigned long)mode, (unsigned long)list,
+			(unsigned long)nent, (unsigned long)sig);
+	if (ret < 0 && errno == ENOTCONN) {
+		aio_init(0);
+		ret = aio(AIOCMD_LIST_IO,
+				(unsigned long)mode, (unsigned long)list,
+				(unsigned long)nent, (unsigned long)sig);
+	}
+	return ret;
+}
+
+#ifndef __USE_FILE_OFFSET64
+#define PAD_AIOCB(aiocb)	*(unsigned int *)&((aiocb)->__aio_pad) = 0;
+#else
+#define PAD_AIOCB(aiocb)
+#endif
+
+int
+aio_read(aiocb_t *aiocb)
+{
+	PAD_AIOCB(aiocb);
+	return(aio_read64((aiocb64_t *)aiocb));
+}
+
+int
+aio_write(aiocb_t *aiocb)
+{
+	PAD_AIOCB(aiocb);
+	return(aio_write64((aiocb64_t *)aiocb));
+}
+
+
+int
+aio_cancel(int fildes, aiocb_t *aiocb)
+{
+	if (aiocb)
+		PAD_AIOCB(aiocb);
+	return(aio_cancel64(fildes, (aiocb64_t *)aiocb));
+}
+
+
+typedef const aiocb64_t * const *CLISTC;
+typedef       aiocb64_t * const	*LISTC;
+
+int
+aio_suspend(const aiocb_t *const list[], int nent, const struct timespec *timeout)
+{
+#ifndef __USE_FILE_OFFSET64
+	int i;
+
+	for (i = 0; i < nent; i++) 
+		if (list[i])
+			PAD_AIOCB(list[i]);
+#endif
+	return(aio_suspend64((CLISTC)list, nent, timeout));
+}
+
+ssize_t
+aio_return(aiocb_t *aiocb)
+{
+	return aio_return64((aiocb64_t *)aiocb);
+}
+
+int
+lio_listio(int mode, aiocb_t *const list[], int nent, sigevent_t *sig)
+{
+#ifndef __USE_FILE_OFFSET64
+	int i;
+
+	for (i = 0; i < nent; i++) 
+		if (list[i])
+			PAD_AIOCB(list[i]);
+#endif
+	return(lio_listio64(mode, (LISTC)list, nent, sig));
+}
+
+void
+aio_pcinvalidate(int fd)
+{
+	if (aio(AIOCMD_PCINVALIDATE, fd, 0, 0, 0)) {
+		if (errno == ENOTCONN) {
+			aio_init(0);
+			if (aio(AIOCMD_PCINVALIDATE, fd, 0, 0, 0) == 0)
+				return;
+		}
+		perror("AIOCMD_PCINVALIDATE");
+		exit(-1);
+	}
+}
